@@ -7,8 +7,8 @@
 ;; Created: 05 Feb 2020
 ;; Keywords: convenience, tools
 ;; Homepage: https://github.com/universal-ctags/citre
-;; Package-Version: 20251015.220
-;; Package-Revision: 300596a34b61
+;; Package-Version: 20260220.1324
+;; Package-Revision: 72375aac87cb
 ;; Package-Requires: ((emacs "26.1"))
 
 ;; This file is NOT part of GNU Emacs.
@@ -83,6 +83,15 @@ If one of these backends are usable in current file,
   :type '(repeat symbol)
   :group 'citre)
 
+(defcustom citre-auto-enable-citre-mode-backends-for-remote '(tags global)
+  "Backends for which `citre-auto-enable-citre-mode' works in remote files.
+Some backends may search for certain files to detect if it's
+usable, which may be slow on a remote host.  If
+`citre-auto-enable-citre-mode' makes opening remote files slow,
+remove problematic backends or set this to nil."
+  :type '(repeat symbol)
+  :group 'citre)
+
 (defcustom citre-auto-enable-citre-mode-modes 'all
   "The major modes where `citre-auto-enable-citre-mode' works.
 If you require `citre-config' in your configuration, then these
@@ -511,9 +520,6 @@ Citre")
 
 ;;;;; Internals
 
-(defvar-local citre-imenu--create-index-function-orig nil
-  "Original value of `imenu-create-index-function' in buffer.")
-
 (defun citre-imenu--classify-tags (tags)
   "Classify TAGS based on the `ext-kind-full' field.
 This creates an alist, its key is `kind' field value, and value
@@ -592,16 +598,14 @@ The returned value is a valid element of the return value of
       (add-hook 'completion-at-point-functions
                 #'citre-completion-at-point nil t))
     (when citre-enable-imenu-integration
-      (setq citre-imenu--create-index-function-orig
-            imenu-create-index-function)
-      (setq imenu-create-index-function #'citre-imenu-create-index-function)))
+      (add-function :before-until (local 'imenu-create-index-function)
+                    #'citre-imenu-create-index-function)))
    (t
     (remove-hook 'xref-backend-functions #'citre-xref-backend t)
     (remove-hook 'completion-at-point-functions
                  #'citre-completion-at-point t)
-    (when citre-enable-imenu-integration
-      (setq imenu-create-index-function
-            citre-imenu--create-index-function-orig)))))
+    (remove-function (local 'imenu-create-index-function)
+                     #'citre-imenu-create-index-function))))
 
 ;;;###autoload
 (defun citre-auto-enable-citre-mode ()
@@ -614,8 +618,12 @@ Put this in `find-file-hook' to automatically enable `citre-mode'
 when opening a file."
   (when (or (eq citre-auto-enable-citre-mode-modes 'all)
             (apply #'derived-mode-p citre-auto-enable-citre-mode-modes))
-    (let ((timeout citre-auto-enable-citre-mode-backend-test-timeout))
-      (cl-dolist (backend citre-auto-enable-citre-mode-backends)
+    (let ((timeout citre-auto-enable-citre-mode-backend-test-timeout)
+          (backends (if (file-remote-p
+                         (or (buffer-file-name) default-directory))
+                        citre-auto-enable-citre-mode-backends-for-remote
+                      citre-auto-enable-citre-mode-backends)))
+      (cl-dolist (backend backends)
         (when (if timeout
                   (with-timeout (timeout) (citre-backend-usable-p backend))
                 (citre-backend-usable-p backend))
